==> GotoBLAS-1.15.tar.gz: GotoBLAS/lapack/potrf/potrf_L_single.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include "common.h"

static FLOAT dm1 = -1.;

#ifndef COMPLEX
#define TRSM_KERNEL   TRSM_KERNEL_RN
#else
#define TRSM_KERNEL   TRSM_KERNEL_RR
#undef SYRK_KERNEL_L
#ifdef XDOUBLE
#define SYRK_KERNEL_L xherk_kernel_LN
#elif defined(DOUBLE)
#define SYRK_KERNEL_L zherk_kernel_LN
#else
#define SYRK_KERNEL_L cherk_kernel_LN
#endif
#endif

#define GEMM_PQ  MAX(GEMM_P, GEMM_Q)
#define REAL_GEMM_R (GEMM_R - GEMM_PQ)

blasint CNAME(long  n, FLOAT *a, long lda, FLOAT *sa, FLOAT *sb){

  long info;
  long bm, bk, i, blocking;
  long is, min_i;
  long ls, min_l;

  FLOAT *sb2 = (FLOAT *)((((long)sb
		    + GEMM_PQ  * GEMM_Q * COMPSIZE * SIZE + GEMM_ALIGN) & ~GEMM_ALIGN)
		  + GEMM_BUFFER_B_OFFSET);

  FLOAT *sub = (FLOAT *)alloca(GEMM_UNROLL_M * GEMM_UNROLL_N * COMPSIZE * SIZE);

#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
  FLOAT *aa;
#endif

  if (n <= DTB_ENTRIES) {
    info = POTF2_L(n, a, lda, sb);
    return info;
  }

  blocking = GEMM_Q;
  if (n <= 4 * GEMM_Q) blocking = n / 4;

  for (i = 0; i < n; i += blocking) {

    bk = n - i;
    if (bk > blocking) bk = blocking;

    info = CNAME(bk, a, lda, sa, sb);
    if (info) return info + i;

    bm = n - i - bk;

    if (bm > 0) {
     
      TRSM_OLTCOPY(bk, bk, a, lda, 0, sb);

      for(ls = 0; ls < bm; ls += REAL_GEMM_R){
	  min_l = bm - ls;
	  if (min_l > REAL_GEMM_R) min_l = REAL_GEMM_R;

	  for (is = ls; is < bm; is += GEMM_P) {
	    min_i = bm - is;
	    if (min_i > GEMM_P) min_i = GEMM_P;
	    
#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
	    if (is < ls + min_l) {
	      aa = sb2 + bk * (is - ls) * COMPSIZE;
	    } else {
	      aa = sa;
	    }
	    GEMM_ITCOPY(bk, min_i, a + (is + bk) * COMPSIZE, lda, aa);
#else
	    GEMM_ITCOPY(bk, min_i, a + (is + bk) * COMPSIZE, lda, sa);
#endif

	    if (ls == 0) {
	      TRSM_KERNEL(min_i, bk, bk, dm1,
#ifdef COMPLEX
			  ZERO,
#endif

#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
			  aa,
#else
			  sa,
#endif
			  sb,
			  a + (is + bk) * COMPSIZE, lda, 0);
	    }

#if (GEMM_UNROLL_M != GEMM_UNROLL_N)
	    if (is < ls + min_l) {
	      GEMM_OTCOPY(bk, min_i, a + (is + bk) * COMPSIZE, lda,
			  sb2 + bk * (is - ls) * COMPSIZE);
	    }
#endif
	    
	    SYRK_KERNEL_L(min_i, min_l, bk, dm1,
#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
			  aa,
#else
			  sa,
#endif
			  sb2,
			  a + (is + bk + (ls + bk) * lda) * COMPSIZE, lda,
			  is - ls, sub);
	  }
      }
    }

    a += (1 + lda) * blocking * COMPSIZE;
  }

  return 0;
}

==> GotoBLAS-1.19.tar.gz: GotoBLAS/lapack/potrf/potrf_L_single.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include "common.h"

static FLOAT dm1 = -1.;

#ifndef COMPLEX
#define TRSM_KERNEL   TRSM_KERNEL_RN
#else
#define TRSM_KERNEL   TRSM_KERNEL_RR
#undef SYRK_KERNEL_L
#ifdef XDOUBLE
#define SYRK_KERNEL_L xherk_kernel_LN
#elif defined(DOUBLE)
#define SYRK_KERNEL_L zherk_kernel_LN
#else
#define SYRK_KERNEL_L cherk_kernel_LN
#endif
#endif

#define GEMM_PQ  MAX(GEMM_P, GEMM_Q)
#define REAL_GEMM_R (GEMM_R - GEMM_PQ)
#define MAX_MN	MAX(GEMM_UNROLL_M, GEMM_UNROLL_N)

blasint CNAME(long  n, FLOAT *a, long lda, FLOAT *sa, FLOAT *sb){

  long info;
  long bm, bk, i, blocking;
  long is, min_i;
  long ls, min_l;

  FLOAT *sb2 = (FLOAT *)((((long)sb
		    + GEMM_PQ  * GEMM_Q * COMPSIZE * SIZE + GEMM_ALIGN) & ~GEMM_ALIGN)
		  + GEMM_BUFFER_B_OFFSET);

  FLOAT *sub = (FLOAT *)alloca(MAX_MN * MAX_MN * COMPSIZE * SIZE);

#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
  FLOAT *aa;
#endif

  if (n <= DTB_ENTRIES) {
    info = POTF2_L(n, a, lda, sb);
    return info;
  }

  blocking = GEMM_Q;
  if (n <= 4 * GEMM_Q) blocking = n / 4;

  for (i = 0; i < n; i += blocking) {

    bk = n - i;
    if (bk > blocking) bk = blocking;

    info = CNAME(bk, a, lda, sa, sb);
    if (info) return info + i;

    bm = n - i - bk;

    if (bm > 0) {
     
      TRSM_OLTCOPY(bk, bk, a, lda, 0, sb);

      for(ls = 0; ls < bm; ls += REAL_GEMM_R){
	  min_l = bm - ls;
	  if (min_l > REAL_GEMM_R) min_l = REAL_GEMM_R;

	  for (is = ls; is < bm; is += GEMM_P) {
	    min_i = bm - is;
	    if (min_i > GEMM_P) min_i = GEMM_P;
	    
#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
	    if (is < ls + min_l) {
	      aa = sb2 + bk * (is - ls) * COMPSIZE;
	    } else {
	      aa = sa;
	    }
	    GEMM_ITCOPY(bk, min_i, a + (is + bk) * COMPSIZE, lda, aa);
#else
	    GEMM_ITCOPY(bk, min_i, a + (is + bk) * COMPSIZE, lda, sa);
#endif

	    if (ls == 0) {
	      TRSM_KERNEL(min_i, bk, bk, dm1,
#ifdef COMPLEX
			  ZERO,
#endif

#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
			  aa,
#else
			  sa,
#endif
			  sb,
			  a + (is + bk) * COMPSIZE, lda, 0);
	    }

#if (GEMM_UNROLL_M != GEMM_UNROLL_N)
	    if (is < ls + min_l) {
	      GEMM_OTCOPY(bk, min_i, a + (is + bk) * COMPSIZE, lda,
			  sb2 + bk * (is - ls) * COMPSIZE);
	    }
#endif
	    
	    SYRK_KERNEL_L(min_i, min_l, bk, dm1,
#if (GEMM_UNROLL_M == GEMM_UNROLL_N)
			  aa,
#else
			  sa,
#endif
			  sb2,
			  a + (is + bk + (ls + bk) * lda) * COMPSIZE, lda,
			  is - ls, sub);
	  }
      }
    }

    a += (1 + lda) * blocking * COMPSIZE;
  }

  return 0;
}
